@@ -10,6 +10,8 @@ import random
 import binascii
 import pkg_resources
 
+from . import *
+
 class ArbitraryBits(object):
     """
     Ridiculous little object to deal with variable-bit-length packed data that
@@ -90,8 +92,30 @@ class BL3Serial(object):
         self.datawrapper = datawrapper
         self.serial_db = datawrapper.serial_db
         self.name_db = datawrapper.name_db
+        self.invkey_db = datawrapper.invkey_db
         self.set_serial(serial)
 
+    def _update_superclass_serial(self):
+        """
+        To be implemented by any superclass which wraps this serial number in
+        a larger object (such as a savegame item structure or profile array).
+        The serial itself will be available in `self.serial`.
+        """
+        pass
+
+    def set_serial(self, serial):
+        """
+        Sets our serial number
+        """
+
+        self.serial = serial
+        (self.decrypted_serial, self.orig_seed) = BL3Serial._decrypt_serial(serial)
+        self.parsed = False
+        self.parts_parsed = False
+        self.can_parse = True
+        self.can_parse_parts = True
+        self.changed_parts = False
+
         # Attributes which get filled in when we parse
         self._version = None
         self._balance_bits = None
@@ -108,15 +132,17 @@ class BL3Serial(object):
         self._level = None
         self._remaining_data = None
 
-    def set_serial(self, serial):
-        """
-        Sets our serial number
-        """
+        # Additional data that gets filled in if we can parse parts
+        self._part_invkey = None
+        self._part_bits = None
+        self._parts = None
+        self._generic_bits = None
+        self._generic_parts = None
+        self._additional_data = None
+        self._num_customs = None
 
-        self.serial = serial
-        (self.decrypted_serial, self.orig_seed) = BL3Serial._decrypt_serial(serial)
-        self.parsed = False
-        self.can_parse = True
+        # Call out to any superclass procedures here
+        self._update_superclass_serial()
 
     @staticmethod
     def _xor_data(data, seed):
@@ -237,6 +263,28 @@ class BL3Serial(object):
             part_val = 'unknown'
         return (part_val, num_bits, part_idx)
 
+    def _get_inv_db_header_part_repeated(self, category, bits, count_bits):
+        """
+        Given the category name `category` and the ArbitraryBits object `bits`,
+        containing serial number data, and `count_bits`, which specifies the
+        number of bits which make up the count of parts to read, returns a
+        tuple containing:
+            1) The number of bits each part in the category takes up
+            2) A list containing tuples with the following:
+                1) The part name
+                2) The numerical index of the part
+        """
+        num_bits = self.serial_db.get_num_bits(category, self._version)
+        parts = []
+        num_parts = bits.eat(count_bits)
+        for _ in range(num_parts):
+            part_idx = bits.eat(num_bits)
+            part_val = self.serial_db.get_part(category, part_idx)
+            if not part_val:
+                part_val = 'unknown'
+            parts.append((part_val, part_idx))
+        return (num_bits, parts)
+
     def _parse_serial(self):
         """
         Parse our serial number, at least up to the level.  We're not going
@@ -255,6 +303,7 @@ class BL3Serial(object):
         self._version = bits.eat(7)
         if self._version > self.serial_db.max_version:
             self.can_parse = False
+            self.can_parse_parts = False
             return
 
         # Now the rest of the data we care about.
@@ -268,13 +317,6 @@ class BL3Serial(object):
                 self._manufacturer_bits,
                 self._manufacturer_idx) = self._get_inv_db_header_part('ManufacturerData', bits)
         self._level = bits.eat(7)
-        self._remaining_data = bits.data
-
-        # At this point, if we were planning on reading parts, we'd read eat six
-        # more bits to find the number of parts, then read in that many, using a
-        # mapping to get to the correct category.  Then another four bits tells
-        # us how many anointments there are, reading from InventoryGenericPartData
-        # after that.  After anointments, I'm not totally sure what's out there.
 
         # Parse out a "short" balance name, for convenience's sake
         self._balance_short = self._balance.split('.')[-1]
@@ -282,26 +324,95 @@ class BL3Serial(object):
         # If we know of an English name for this balance, use it
         self._eng_name = self.name_db.get(self._balance_short)
 
-        # Mark down that we're parsed, now.
+        # Mark down that we've parsed the basic info (we have enough to level up
+        # gear at this point)
         self.parsed = True
 
+        # Make a note of our remaining data - if we re-save without any parts
+        # changes, we can just use this rather than reconstructing the whole
+        # serial.
+        self._remaining_data = bits.data
+
+        # Now let's see if we can parse parts
+        self._part_invkey = self.invkey_db.get(self._balance)
+        if self._part_invkey is None:
+            self.can_parse_parts = False
+        else:
+
+            # Let's assume at first that we're going to correctly parse all this
+            self.parts_parsed = True
+
+            # Read parts
+            (self._part_bits, self._parts) = self._get_inv_db_header_part_repeated(
+                    self._part_invkey, bits, 6)
+
+            # Read generics (anointments+mayhem)
+            (self._generic_bits, self._generic_parts) = self._get_inv_db_header_part_repeated(
+                    'InventoryGenericPartData', bits, 4)
+
+            # Read additional data (no idea for the most part; some item "wear"
+            # is in here, we think.  Maybe other stuff, too?)
+            additional_count = bits.eat(8)
+            self._additional_data = []
+            for _ in range(additional_count):
+                self._additional_data.append(bits.eat(8))
+
+            # Read in "customization" parts; this presumably used to be
+            # trinkets+weaponskins, but was removed at some point.  If we
+            # have anything but 0 in here, we're going to force `can_parse_parts`
+            # to false, 'cause we don't know how many bits these things
+            # might take if they're present.
+            self._num_customs = bits.eat(4)
+            if self._num_customs != 0:
+                self.parts_parsed = False
+                self.can_parse_parts = False
+
+            # And read in our remaining data.  If there's more than 7 bits
+            # left, we've done something wrong, because it should only be
+            # zero-padding after all the "real" data is in place.
+            if len(bits.data) > 7:
+                self.parts_parsed = False
+                self.can_parse_parts = False
+            elif '1' in bits.data:
+                # This is supposed to only be zero-padding at the moment, if
+                # we see something else, abort
+                self.parts_parsed = False
+                self.can_parse_parts = False
+            else:
+                # Okay, we're good!  Don't bother saving the remaining 0 bits.
+                pass
+
     def _deparse_serial(self):
         """
         De-parses a serial; used after we make changes to the data that gets
-        pulled out during `_parse_serial`.  At the moment, that's only going
-        to be item level changes.  Will update the serial in the protobuf as
-        well, and set the object to trigger a re-parse if anything else needs
-        to read more.
-
-        This is all pretty inefficient -- really we should just write over the
-        bits values with the new values, in-place.  But whatever, this'll do
-        for now.
+        pulled out during `_parse_serial`.  At the moment, that's both level
+        changes and mayhem level changes.  Will end up calling out to the
+        superclass's `_update_superclass_serial` to propagate the serial change
+        to whatever containing structure needs it, and set the object to trigger a
+        re-parse if anything else needs to read more.  That's probably overkill
+        and makes this technically quite inefficient, especially when making
+        multiple edits to the same item, but given the scale of processing,
+        we'll probably be fine.
         """
 
         if not self.can_parse:
             return
 
-        # Construct the new item data
+        if self.changed_parts:
+            # If we changed any parts, re-save using the latest serial version,
+            # which means that we'll have to figure out new bit lengths for
+            # everything.  I'm not doing this *all* the time because I like
+            # changing as little as possible when doing these edits, and this
+            # way we can do stuff like change the level of an item without
+            # having to re-encode its parts.
+            self._version = self.serial_db.max_version
+            self._balance_bits = self.serial_db.get_num_bits('InventoryBalanceData', self._version)
+            self._invdata_bits = self.serial_db.get_num_bits('InventoryData', self._version)
+            self._manufacturer_bits = self.serial_db.get_num_bits('ManufacturerData', self._version)
+            self._part_bits = self.serial_db.get_num_bits(self._part_invkey, self._version)
+            self._generic_bits = self.serial_db.get_num_bits('InventoryGenericPartData', self._version)
+
+        # Construct a new header
         bits = ArbitraryBits()
         bits.append_value(128, 8)
         bits.append_value(self._version, 7)
@@ -309,13 +420,43 @@ class BL3Serial(object):
         bits.append_value(self._invdata_idx, self._invdata_bits)
         bits.append_value(self._manufacturer_idx, self._manufacturer_bits)
         bits.append_value(self._level, 7)
-        bits.append_data(self._remaining_data)
+
+        # Arguably we should *always* re-encode parts, if we're able to, just so this
+        # function is less complex.  For now I'm keeping it like this, though.
+
+        if self.changed_parts:
+            # If we've changed parts, just write out everything again.  First parts
+            bits.append_value(len(self._parts), 6)
+            for (part_val, part_idx) in self._parts:
+                bits.append_value(part_idx, self._part_bits)
+
+            # Then generics
+            bits.append_value(len(self._generic_parts), 4)
+            for (part_val, part_idx) in self._generic_parts:
+                bits.append_value(part_idx, self._generic_bits)
+
+            # Then additional data
+            bits.append_value(len(self._additional_data), 8)
+            for value in self._additional_data:
+                bits.append_value(value, 8)
+
+            # Then our number of customs (should always be zero)
+            bits.append_value(self._num_customs, 4)
+
+        else:
+            # Otherwise, we can re-use our original remaining data
+            bits.append_data(self._remaining_data)
+
+        # Read the serial back out of our structure
         new_data = bits.get_data()
 
         # Encode the new serial (using seed 0; unencrypted)
         new_serial = BL3Serial._encrypt_serial(new_data, 0)
 
         # Load in the new serial (this will set `parsed` to `False`)
+        # It bothers me that I've just done an `_encrypt_serial` in the
+        # previous statement, when this call to `set_serial` will just
+        # turn right around and decrypt it again.  Alas.
         self.set_serial(new_serial)
 
     @property
@@ -383,6 +524,7 @@ class BL3Serial(object):
         # Set the level and trigger a re-encode of the serial
         self._level = value
         self._deparse_serial()
+        self._update_superclass_serial()
 
     def get_serial_number(self, orig_seed=False):
         """
@@ -415,6 +557,101 @@ class BL3Serial(object):
         encoded = new_data[4:-1]
         return base64.b64decode(encoded)
 
+    @property
+    def mayhem_level(self):
+        """
+        Returns the current Mayhem level of the item, with `0` signifying
+        that there is no Mayhem level present, and `None` signifying that
+        the Mayhem level could not be parsed (due to being unable to parse the
+        item parts)
+        """
+        if not self.parsed or not self.parts_parsed:
+            self._parse_serial()
+            if not self.can_parse or not self.can_parse_parts:
+                return None
+        # Given the presence of item editors, there could possibly be more
+        # than one Mayhem part present in a serial (though they don't seem
+        # to stack at all, so doing so would be pointless).  We'll just
+        # abort processing as soon as we find one, which I suspect is likely
+        # what the game does, too.
+        for part_name, part_idx in self._generic_parts:
+            if part_name.lower() in mayhem_part_lower_to_lvl:
+                return mayhem_part_lower_to_lvl[part_name.lower()]
+        return 0
+
+    def can_have_mayhem(self):
+        """
+        Returns `True` if this is an item type which can have a mayhem level,
+        or `False` otherwise.  Will also return `False` if we're unable to
+        parse parts for the item.
+        """
+        if not self.parsed or not self.parts_parsed:
+            self._parse_serial()
+            if not self.can_parse or not self.can_parse_parts:
+                return False
+        return self._invdata.lower() in mayhem_invdata_lower_types
+
+    @mayhem_level.setter
+    def mayhem_level(self, value):
+        """
+        Sets the given mayhem level on the item.  Returns `True` if we were
+        able to do so, or `False` if not.
+        """
+        # The call to `can_have_mayhem` will parse the serial if possible,
+        # so we'll be all set.
+        if not self.can_have_mayhem():
+            return False
+
+        # Don't forget to set this
+        self.changed_parts = True
+
+        # First grab a list of any non-Mayhem parts (should just be anoints)
+        new_parts = []
+        for idx, (part_name, part_idx) in enumerate(self._generic_parts):
+            if part_name.lower() not in mayhem_part_lower_to_lvl:
+                new_parts.append((part_name, part_idx))
+
+        # Now add our new one in
+        if value > 0:
+            new_mayhem_part = self.serial_db.get_part_index(
+                    'InventoryGenericPartData',
+                    mayhem_lvl_to_part[value],
+                    )
+            if new_mayhem_part is None:
+                return False
+            else:
+                new_parts.append((mayhem_lvl_to_part[value], new_mayhem_part))
+
+        # Aaaand assign our list of generic parts back
+        self._generic_parts = new_parts
+
+        # Re-serialize
+        self._deparse_serial()
+        self._update_superclass_serial()
+
+        # return!
+        return True
+
+    def get_level_eng(self):
+        """
+        Returns an English representation of our level, including Mayhem level,
+        suitable for reporting to a user.
+        """
+        # First, regular level
+        level = self.level
+        if level is None:
+            return 'unknown lvl'
+        to_ret = 'level {}'.format(level)
+
+        # Then Mayhem
+        mayhem_level = self.mayhem_level
+        if mayhem_level is None:
+            return '{}, mayhem unknown'.format(to_ret)
+        elif mayhem_level > 0:
+            return '{}, mayhem {}'.format(to_ret, mayhem_level)
+        else:
+            return to_ret
+
 class InventorySerialDB(object):
     """
     Little wrapper to provide access to our inventory serial number DB
@@ -424,6 +661,7 @@ class InventorySerialDB(object):
         self.initialized = False
         self.db = None
         self._max_version = -1
+        self.part_cache = {}
 
     def _initialize(self):
         """
@@ -480,6 +718,23 @@ class InventorySerialDB(object):
             else:
                 return self.db[category]['assets'][index-1]
 
+    def get_part_index(self, category, part_name):
+        """
+        Find the correct index to use for the given `part_name`, inside the given
+        `category`.  Will return `None` if the part cannot be found.
+        """
+        if category not in self.part_cache:
+            self.part_cache[category] = {}
+        if part_name not in self.part_cache[category]:
+            for idx, asset_part_name in enumerate(self.db[category]['assets']):
+                if part_name == asset_part_name:
+                    self.part_cache[category][part_name] = idx+1
+                    return idx+1
+        if part_name in self.part_cache[category]:
+            return self.part_cache[category][part_name]
+        else:
+            return None
+
 class BalanceToName(object):
     """
     Little wrapper to provide access to a mapping from Balance names (actually
@@ -519,6 +774,42 @@ class BalanceToName(object):
         else:
             return None
 
+class BalanceToInvKey(object):
+    """
+    Little wrapper to provide access to a mapping from Balance names to
+    the inventory key that we'd need to use to read its parts out.
+    """
+
+    def __init__(self):
+        self.initialized = False
+        self.mapping = None
+
+    def _initialize(self):
+        """
+        Actually read in our data.  Not doing this automatically because I
+        only want to do it if we're doing an operation which requires it.
+        """
+        if not self.initialized:
+            with lzma.open(io.BytesIO(pkg_resources.resource_string(
+                    __name__, 'resources/balance_to_inv_key.json.xz'
+                    ))) as df:
+                self.mapping = json.load(df)
+            self.initialized = True
+
+    def get(self, balance):
+        """
+        Returns the inventory key for the given balance, if we can.
+        """
+        if not self.initialized:
+            self._initialize()
+        if '.' not in balance:
+            balance = '{}.{}'.format(balance, balance.split('/')[-1])
+        balance = balance.lower()
+        if balance in self.mapping:
+            return self.mapping[balance]
+        else:
+            return None
+
 class DataWrapper(object):
     """
     Weird little metaclass which just has an instance of each of our file-backed
@@ -531,4 +822,5 @@ class DataWrapper(object):
     def __init__(self):
         self.serial_db = InventorySerialDB()
         self.name_db = BalanceToName()
+        self.invkey_db = BalanceToInvKey()
 
